#include <dsetup.h>
#include <msiquery.h>
#include <Shlwapi.h>
#include <cstdio>

static PMSIHANDLE hActionRec;
static MSIHANDLE hInstall;

static DWORD __stdcall callback(DWORD Reason, DWORD MsgType, char * pszMessage, char * pszName, void * pInfo)
{   
    static const char* pszInstalling = "Installing DirectX:";
    if(!hActionRec)
    {
        return IDOK;
    }

    if(Reason==DSETUP_CB_MSG_PROGRESS)
    {   
        char szDesc[MAX_PATH+50];
        const char* pszPhaseText = "";
        
        DSETUP_CB_PROGRESS* pProgress = reinterpret_cast<DSETUP_CB_PROGRESS*>(pInfo);
        switch(pProgress->dwPhase)
        {
        case DSETUP_INITIALIZING:
            pszPhaseText = "Initializing";
            break;
        case DSETUP_EXTRACTING:
            pszPhaseText = "Extracting";
            break;
        case DSETUP_COPYING:
            pszPhaseText = "Copying";
            break;
        case DSETUP_FINALIZING:
            pszPhaseText = "Finalizing";
            break;
        }        

        if((pProgress->dwPhase == DSETUP_COPYING || pProgress->dwPhase == DSETUP_EXTRACTING) && !pszName) //There are copy progress messages with no file name; worthless for us
        {
            return IDOK;
        }

        if(pszName)
        {
            sprintf_s(szDesc,_countof(szDesc),"%s %s %s (%d of %d)",pszInstalling,pszPhaseText,pszName,pProgress->dwInPhaseProgress,pProgress->dwInPhaseMaximum);            
        }
        else
        {
            sprintf_s(szDesc,_countof(szDesc),"%s %s",pszInstalling, pszPhaseText);
        }        

        //The standard Wix Mondo UI doesn't have an ActionData text label, so just change the action name
        MsiRecordSetStringA(hActionRec, 1, "InstallDX"); //Action name
        MsiRecordSetStringA(hActionRec, 2, szDesc); //Description       
        MsiProcessMessage(hInstall, INSTALLMESSAGE_ACTIONSTART, hActionRec);
    }
    return IDOK;
}

/**
Read the Custom Action Data for this CA
*/
static bool GetCustomActionData(MSIHANDLE hInstall, wchar_t* pszOut)
{
    DWORD dwLength=MAX_PATH;

    if(MsiGetProperty(hInstall, L"CustomActionData", pszOut, &dwLength) != ERROR_SUCCESS)
    {	     
        return false;
    }
    return true;
}

extern "C" __declspec(dllexport) UINT InstallDirectX(MSIHANDLE hInstall)
{    
    ::hInstall = hInstall;

    //Get directory where dependency DLL files are installed, and load them manually before the system tries to do this.
    //FOR THIS TO WORK, THE DLLS MUST BE SET TO DELAY-LOADED IN THE PROJECT LINKER OPTIONS!   
    wchar_t szInstallDir[MAX_PATH];
    GetCustomActionData(hInstall, szInstallDir);
    wchar_t szDSetupDllPath[MAX_PATH];
    PathCombine(szDSetupDllPath,szInstallDir,L"dsetup.dll");
 
    HMODULE pDSetupDll = LoadLibrary(szDSetupDllPath);
    if(!pDSetupDll)
    {        
        return EXIT_FAILURE;
    }

    if (MsiGetMode(hInstall,MSIRUNMODE_SCHEDULED) == FALSE)
    {
        return EXIT_FAILURE; //Must be deferred CA
    }

    //Create MSI process status
    hActionRec = MsiCreateRecord(3);
    if(hActionRec == NULL)
    {
        return EXIT_FAILURE;
    }

    //Install DirectX
    DirectXSetupSetCallback(callback);
    INT result = DirectXSetup(0,szInstallDir,DSETUP_DIRECTX);    
    MsiCloseHandle(hActionRec);
    if(result != DSETUPERR_SUCCESS && result != DSETUPERR_SUCCESS_RESTART)
    {
        return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}

